@@ -4,8 +4,12 @@ SystemPlayer::SystemPlayer(QObject *parent) :
     QObject(parent)
 {
 #ifdef Q_WS_MAEMO_5
-    m_renderer = new MafwRendererAdapter(this);
+    m_state = SystemPlayer::UnknownState;
+    m_renderer = new MafwRendererAdapter();
     m_Playlist = new MafwPlaylistAdapter(this, m_renderer);
+    m_Tracker = new MafwSourceAdapter("Mafw-Tracker-Source");
+
+    connect(m_Tracker, SIGNAL(signalMetadataResult(QString,GHashTable*,QString)), this, SLOT(onGotMafwTrackerMetadata(QString,GHashTable*,QString)));
     connect(m_Playlist, SIGNAL(playlistChanged()), this, SLOT(onPlaylistChanged()));
     connect(m_renderer, SIGNAL(signalGetPosition(int,QString)), this, SLOT(onGotPosition(int,QString)));
     connect(m_renderer, SIGNAL(mediaChanged(int,char*)), this, SLOT(mafwMediaChanged(int,char*)));
@@ -77,7 +81,12 @@ void SystemPlayer::getPosition()
 void SystemPlayer::getCurrentMetadata()
 {
 #ifdef Q_WS_MAEMO_5
-    m_renderer->getCurrentMetadata();
+    qDebug() << "getCurrentMetadata:" << m_state << m_ObjectId;
+    if (!m_ObjectId.isEmpty() &&
+        (m_state == SystemPlayer::StoppedState || m_state == SystemPlayer::UnknownState))
+        mafwMediaChanged(0, m_ObjectId.toUtf8().data());
+    else
+        m_renderer->getCurrentMetadata();
 #endif
 }
 
@@ -124,14 +133,17 @@ void SystemPlayer::mafwStateChanged(int mafwState)
 void SystemPlayer::onGotMafwState(MafwPlaylist*, uint, MafwPlayState state, const char* objectId, QString)
 {
     qDebug() << "onGotMafwState: objectId" << objectId;
+
+    bool update = m_ObjectId.isEmpty();
+
+    m_ObjectId = QString::fromUtf8(objectId);
     this->mafwStateChanged(state);
+    if (update)
+        getCurrentMetadata();
 }
 
-void SystemPlayer::onGotMafwMetadata(GHashTable* table,QString objectId,QString)
+void SystemPlayer::onGotMafwMetadata(GHashTable* table,QString,QString)
 {
-    /*if (m_ObjectId == objectId)
-        return;*/
-    m_ObjectId = objectId;
     QMap <QString, QVariant> metadata;
     GValue *v;
     v = mafw_metadata_first(table, MAFW_METADATA_KEY_ARTIST);
@@ -143,8 +155,20 @@ void SystemPlayer::onGotMafwMetadata(GHashTable* table,QString objectId,QString)
     v = mafw_metadata_first(table, MAFW_METADATA_KEY_ALBUM);
     QString album = v ? QString::fromUtf8(g_value_get_string(v)) : tr("Unknown album");
 
+    QString art = "";
     v = mafw_metadata_first(table, MAFW_METADATA_KEY_RENDERER_ART_URI);
-    QString art = v ? QString::fromUtf8(g_value_get_string(v)) : "";
+    if(v != NULL) {
+        art = QString::fromUtf8(g_value_get_string(v));
+        qDebug() << "MAFW_METADATA_KEY_ALBUM_ART_URI:" << art;
+    } else {
+        v = mafw_metadata_first(table, MAFW_METADATA_KEY_ALBUM_ART_URI);
+        if(v != NULL) {
+            const gchar* file_uri = g_value_get_string(v);
+            gchar* filename = g_filename_from_uri(file_uri, NULL, NULL);
+            art = QString::fromUtf8(filename);
+            qDebug() << "MAFW_METADATA_KEY_ALBUM_ART_URI:" << art;
+        }
+    }
 
     metadata.insert(SYSTEM_PLAYER_SONG_TITLE, songName);
     metadata.insert(SYSTEM_PLAYER_SONG_ALBUM, album);
@@ -162,7 +186,17 @@ void SystemPlayer::onPlaylistChanged()
 void SystemPlayer::mafwMediaChanged(int, char* objectId)
 {
     qDebug() << "Media changed:" << objectId;
-    //m_renderer->getCurrentMetadata();
+    m_ObjectId = QString::fromUtf8(objectId);
+    m_Tracker->getMetadata(objectId, MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE,
+                           MAFW_METADATA_KEY_ALBUM,
+                           MAFW_METADATA_KEY_ARTIST,
+                           MAFW_METADATA_KEY_ALBUM_ART_URI,
+                           MAFW_METADATA_KEY_RENDERER_ART_URI));
+}
+
+void SystemPlayer::onGotMafwTrackerMetadata(QString objectId, GHashTable* metadata_keys, QString error)
+{
+    onGotMafwMetadata(metadata_keys, objectId, error);
 }
 
 #endif
