@@ -17,7 +17,11 @@ public:
     static QString currentProfile();
     static void setProfile(QString profileName);
 
+private:
+    static void profileChanged_cb(const char *profile, void *user_data);
+
 signals:
+    void profileChanged(QString profileName);
 
 public slots:
 
